/* preloading */
(function() {
    (new Image).src = 'images/edit_note.png';
})();

// {%TRANSLATION%}
var strings = {
    WELCOME_MSG_POST : _('Type your note here…'),
    WELCOME_MSG_AUTHOR : _('Your name'),
    WELCOME_MSG_EMAIL : _('Your e-mail address (hidden)'),
    EMPTY_ERR_MSG : _('Empty notes are not allowed.'),
    NAME_TOO_LONG : _('Please use a shorter nickname.'),
    EMAIL_TOO_LONG : _('Please use a shorter e-mail address.'),
    
    // This message appears in confirm window (OK, Cancel) on guest's computer.
    // (onwer: Start a service; guest: visit it, put a note; owner: restart;
    // guest: try to post a note or delete one to see the message appear.)
    SESSION_LOST: _('Sorry! Your session is lost. Reload to start a new session?'),
    
    // I don't think this can ever happen. But just in case:
    // A DATA_MALFORMED error (see script.js) is a fatal-error which
    // prevents the service from working. What is done know, is that
    // DATA_MALFORMED<br/>TRY_LATER appear on a white screen.
    // Again, this should not happen unless some1 manually breaks their own frigde.
    TRY_LATER : _('Please try again later.')
};

function Wall() {

    var conf = {
        ACTION : document.forms.shout.action?document.forms.shout.action:'#',
        POLL_INTERVAL : 25    // in sec
    };

    var revision = (new Date()).valueOf();
    var isReadonly = false;
    
    var elCapacity = document.getElementById('capacity');
    var isAdmin = !!elCapacity;
    if (isAdmin) {
        var capacity = elCapacity.value;
    }
    
    var form = document.forms.shout;
    var shoutbox = document.getElementById('shoutBox');
    
    var token = form.token.value;
    
    // http://wiki.codetalks.org/wiki/index.php/Docs/Keyboard_navigable_JS_widgets#Solution_1:_tabindex
    function addTabIndex() {
        var entries = document.getElementsByClassName('entry');
        
        for( var i = 0, entry; entry = entries[i]; i++ ) {
            entry.tabIndex = 0;
            entry.onfocus = (function(entry){ return function() {
                var form = entry.getElementsByTagName('form')[0];
                var button = entry.getElementsByTagName('button')[0];
                if( !button ) { return; }
                // it basically imitates the same behaviour as form:hover
                form.style.display = 'block';
                button.focus();
                button.onblur = function() {
                    button.onblur = undefined;
                    form.style.display = '';
                }
            }})(entry);
        }
    }
    
    function addRemoveListeners() {
        var trashButtons = document.getElementsByName('delete');
        
        for (var i=0, button; button = trashButtons[i]; ++i) {
            if (button.onclick !== sendRemoveRequest) { 
                button.onclick = sendRemoveRequest; 
            }
        }
    };

    function sendRemoveRequest(e) {
        e = e || window.event;
        e.preventDefault = e.preventDefault || function() { 
            this.returnValue = false; 
        };

        var sBody = 'remove='+this.previousSibling.value+'&token='+this.nextSibling.value;

        keepOn(sBody);

        e.preventDefault();
        return;
    };    
    
    function refreshDates() {
        var noteForms = document.getElementsByClassName('note_form');
        var now = new Date;
        for (var i=0, ntForm; ntForm = noteForms[i]; ++i)
            getElementsByClassName.call(ntForm.parentNode,'date')[0].innerHTML
                = (new Date(parseInt(ntForm.remove.value))).happens(now);
    };
    
    function removeNotes(remove) {
        for (var s = 0, sDate; sDate = remove[s]; ++s) {

            var noteForms = document.getElementsByClassName('note_form');
            for (var i = 0, ntForm; ntForm = noteForms[i]; ++i) {
                if (ntForm.remove.value == sDate) {

                    // against IE memory leaks
                    // if (ntForm['delete']) { 
                    //     ntForm['delete'].onclick = null; 
                    // }
                    purge(ntForm.parentNode, true);

                    ntForm.parentNode.parentNode.removeChild(
                        ntForm.parentNode
                    );
                    break;
                }
            }
        }
    };
    
    function onChangeReadonly() {
        if (isAdmin) { return };
        
        for (var i = 0, el; el = form.elements[i]; ++i) {
            el.disabled = isReadonly;
        }
        
        // js - animate in future releases?
        document.getElementById('add_note_wrapper').style.display
            = isReadonly ? 'none' : 'block'; 
    };
    
    function onSuccessfulResponse(xhr) {

        // --- HEADERS
        
        // ------ Fatal-error, Error
        var error;
        
        // There are errors that should prevent auto-request; these
        // will be signified be 'Fatal-error' header.
        if (error = xhr.getResponseHeader('X-Fatal-error')) {
            clearTimeout(oTimeout);
            xhr.abort();
        
            document.body.innerHTML = error + '<br />' + strings.TRY_LATER;
            return;
        }
        if (error = xhr.getResponseHeader('X-Error')) {
            if (error == 'Unauthenticated!') {
                var reload = confirm(strings.SESSION_LOST);
                if (reload) {window.location.reload();}
            }
            else {alert(error);}
        }
        
        // ------ Revision, PrevRevision
        var prevRevision = parseInt(xhr.getResponseHeader('X-Prev-revision'));
        var newRevision = parseInt(xhr.getResponseHeader('X-Revision'));
        
        // when admin closes the service, FF receives malformed headers:
        if (isNaN(newRevision) || isNaN(prevRevision)) { return }
        
        // ------ Revision, PrevRevision
        // if it's first request or readonly attribute changed - change 
        // the form appropriately
        var hdrReadonly = (xhr.getResponseHeader('X-Readonly') == 'false')
            ? false : true;
        if (revision > newRevision || hdrReadonly != isReadonly) {
            isReadonly = hdrReadonly;
            onChangeReadonly()
        }
        
        // ------ Capacity
        if (isAdmin) {
            var newCapacity = parseInt(xhr.getResponseHeader('X-Capacity'));
            if (newCapacity != capacity && !isNaN(newCapacity)) {
                capacity = newCapacity;
                elCapacity.value = capacity;
            }
        }
        
        // ------ Revision, PrevRevision
        var remove = xhr.getResponseHeader('X-Remove');
        if (remove) { removeNotes(remove.split(',')); }
        
        // --- BODY

        // if it's fresh request or there where no other changes meanwhile
        // just append the response; otherwise parse the whole content
        if (revision >= prevRevision) {
        
            // var frag = document.createDocumentFragment();
            // for (var i = 0, child; child = shoutbox.childNodes[i]; ) {
            //     frag.appendChild(child);
            // };
            // shoutbox.innerHTML = xhr.responseText;
            // for (var i = 0, child; child = frag.childNodes[i]; ) {
            //     shoutbox.appendChild(child);
            // }
            
            // or:
            purge(shoutbox, true, ['onclick']);
            shoutbox.innerHTML = xhr.responseText + shoutbox.innerHTML;    
        } else {
            purge(shoutbox, true, ['onclik']);
            shoutbox.innerHTML = xhr.responseText;
        }
        
        addTabIndex();
        addRemoveListeners();
        
        // ---
        
        revision = newRevision;
        keepOn();
    };
    
    
    var oTimeout = null;
    var xhr = { abort : function(){} };
    function keepOn(sBody) {
        clearTimeout(oTimeout);
        
        xhr.abort();
        
        refreshDates();
        
        sBody = sBody || null;
        
        xhr = sendRequest(sBody);
        oTimeout = setTimeout( keepOn , conf.POLL_INTERVAL*1000 );
    };
    
    function sendRequest(sBody, callback) {

        var xhr = new XMLHttpRequest();
        
        xhr.onreadystatechange = function() {
            // this won't work for ie6; use explicit variable: xhr
            var self = xhr; // this
            if (self.readyState == 4) {
                if (self.status == 200) {
                    onSuccessfulResponse(self);
                }
            }
        };

        xhr.open('POST', location.href, true);
        
        xhr.setRequestHeader('X-Revision', revision );

        xhr.setRequestHeader(
            'Content-Type', 
            'application/x-www-form-urlencoded; charset=utf-8'
        );

        xhr.send(sBody);
        
        return xhr;
    };

    this.keepOn = keepOn;
    
    function __init__() {
        refreshDates();
        addTabIndex();
        addRemoveListeners();
    };
    
    __init__();
};


//--- event functions out of scope

function onBlur(msg, func) {
    return function() {
        if (!this.value) {
            this.className = 'empty';
            this.value = msg;
            if (func) {func.call(this);};
        }
    }
};

function onFocus() {        
    if (this.className == 'empty') {
        this.className = '';
        this.value = '';
    }
};

addEvent(window, 'load', function() {

    window.oWall = new Wall();
    
    //Max post length used in the server script.js too.
    var conf = {
        MAX_POST_LENGTH : 60,
        FONT_MIN : 30,
        FONT_MAX : 60
    };

    // --- CREATE & FIND
    
    // html elements
    var charsLeft = document.getElementById('charsLeft'),
        textarea = document.forms.shout.post,
        email = document.getElementById('email'),
        author = document.getElementById('author'),
        addNote = document.getElementById('add_note'),
        shoutForm = document.forms.shout;
    
    var lightboxShade = document.createElement('div');    
    lightboxShade.id = 'lightbox_shade';
    document.body.appendChild(lightboxShade);
    // functions
    
    function editModeOn() {

        // lightboxShade.style.display = 'block';
        // shoutForm.style.display = 'block';
        lightboxShade.style.opacity = 0;
        shoutForm.style.opacity = 0;
        lightboxShade.style.display = 'block';
        shoutForm.style.display = 'block';
        var start = (new Date).valueOf();
        var duration = 400;
        var to = 50;
		var textArea = document.getElementById('postTextArea');
		textArea.focus();
        (function() {
            var elapsed = (new Date).valueOf() - start;
            var opacity = elapsed / duration;
            opacity = opacity > 1 ? 1 : opacity;

            lightboxShade.style.opacity = opacity/2;
            shoutForm.style.opacity = opacity;

            if (opacity < 1) {
                setTimeout(arguments.callee, to);
            }
        })();
    
    };
    
    function editModeOff() {
        // lightboxShade.style.display = 'none';
        // shoutForm.style.display = 'none';
    
        var start = (new Date).valueOf();
        var duration = 400;
        var to = 50;
        (function() {
            var elapsed = (new Date).valueOf() - start;
            var opacity = 1 - elapsed / duration;
            opacity = opacity < 0 ? 0 : opacity;

            lightboxShade.style.opacity = opacity/2;
            shoutForm.style.opacity = opacity;

            if (opacity > 0) {
                setTimeout(arguments.callee, to);
            } else {
                lightboxShade.style.display = 'none';
                shoutForm.style.display = 'none';
            }
        })();
    
    };
    
    // --- INIT
    
    // inputs onblur
    author.onblur = onBlur(strings.WELCOME_MSG_AUTHOR);
    email.onblur = onBlur(strings.WELCOME_MSG_EMAIL);
    
    //> leak evt
    textarea.onblur = onBlur(strings.WELCOME_MSG_POST, function() {
        charsLeft.innerHTML = _('{charactersLeft} characters left.', {charactersLeft: conf.MAX_POST_LENGTH});
        this.setFont();
    });

    // inputs onfocus
    author.onfocus = email.onfocus = onFocus;
    //> leak evt
    textarea.onfocus = function() {
        onFocus.call(this);
        this.onclick();
    };
    
    // font resizing
    var fontSetter = window.getFontSetter(textarea, conf.FONT_MIN, conf.FONT_MAX);
    //> leak prop
    textarea.setFont = function() {
        var fontSize = fontSetter.computeFont(
            this.value.replace(/\n/g,'<br/>').replace(/  /g, ' &nbsp;')
        );
        this.style.fontSize = fontSize + 'px';
        return fontSize;
    };
    
    // update 'chars left' div and font size
    //var scrollTimeout;
    textarea.onchange = textarea.onclick = textarea.onkeyup = function(e) {
        clearTimeout(arguments.callee.scrollTimeout);
        
        // ctrl enter submit
        e = e || window.event;
        if (e.ctrlKey && e.keyCode == 13) {
            sendPost.call(shoutForm, {preventDefault : function(){}});
            return;
        };
        
        var normalized = this.value;
        if (!window.opera) {
            normalized = this.value.replace(/\r\n|\r|\n/g, "\r\n");
        }
        var diff = normalized.length - this.value.length;
        
        if (normalized.length > conf.MAX_POST_LENGTH) {

            this.value = this.value.substring(0, conf.MAX_POST_LENGTH);
            normalized = this.value.replace(/\r\n|\r|\n/g, "\r\n");
            
            while (normalized.length > conf.MAX_POST_LENGTH) {
                if (this.value.substring(this.value.length-2, this.value.length)
                    === '\r\n') {
                    this.value = this.value.substring(0, this.value.length-2);
                } else {
                    this.value = this.value.substring(0, this.value.length-1);
                }
                normalized = this.value.replace(/\r\n|\r|\n/g, "\r\n");
            }
        }
        
        var iLeft = conf.MAX_POST_LENGTH - normalized.length;
        charsLeft.innerHTML = _('1 character left.', '{charactersLeft} characters left.', {counter: iLeft, charactersLeft: iLeft});
        charsLeft.style.color = 'rgb('
            +(255-(iLeft/conf.MAX_POST_LENGTH)*255 >> 0)
            +',0,'
            +((iLeft/conf.MAX_POST_LENGTH)*255 >> 0)
            +')';

        var fontSize = this.setFont();

        // max lines message ?
        var maxlines = 1, lines = 0;
        if (fontSize == conf.FONT_MIN) {
        
            var styles = window.getComputedStyle(this);
            lineHeight = parseInt(styles.getPropertyValue('line-height'));

            if (isNaN(lineHeight)) { lineHeight = 1.5 * fontSize };
            
            maxlines = (this.clientHeight / lineHeight) >> 0;
            lines = (this.scrollHeight / lineHeight) >> 0;
            
            if (lines > maxlines) {
                var msg = '&nbsp;&nbsp;' + _('{maxLines} lines allowed.', {maxLines: maxlines});
                charsLeft.innerHTML += msg;
            }
        }
        
        // convenience issue for Opera (it scrolls stuff for some reason)
        if (maxlines >= lines) {
            var self = this;
            arguments.callee.scrollTimeout 
                = setTimeout(function(){self.scrollTop = 0;}, 999);
        }
        
        // disable sent (if empty)
        // document.getElementById('send').disabled = !this.value.trim();
        // i think diffrent styling would be sufficient ('empty post are
        // not allowed' will pop up then');

    };
    
/*
    // TAB key make 4 spaces
    textarea.getSelectionRange = getSelectionRange;
    if (!textarea.setSelectionRange) {
        textarea.setSelectionRange = window.setSelectionRange;
    }
    textarea.onkeydown = function(e) {
        e = e || window.event;
        if (e.keyCode == 9) { // tab -> 4 spaces
            var range = this.getSelectionRange();
            var start = range[0];
            var end = range[1];

            this.value = this.value.substr(0, start)
                + '    ' + this.value.substr(end);
                
            var self = this;
            setTimeout( function() { 
                self.focus();
                self.setSelectionRange(start+4, start+4);
            }, 200 );
        }
    };
*/
    // animate add_note button click
    //> leak evt
    addNote.onmousedown = function() {
        this.parentNode.parentNode.style.top = '0px';
    };
    addNote.onmouseup = function() {
        this.parentNode.parentNode.style.top = '';
    };
    
    // show/hide new note editing
    addNote.onclick = editModeOn;
    lightboxShade.onclick = editModeOff;
    document.getElementById('cancel').onclick = editModeOff;

    // ie bug: Enter won't submit
    // NOTICE: submit button must be there for this to work!
    // (it can't be 'display: none' nor 'visibility: hidden')
    author.onkeydown = email.onkeydown = function (e) {
        if (!e && window.event) { 
            if (window.event.keyCode == 13) {
                sendPost.call(shoutForm, {preventDefault : function(){}});
                return false;
            }
        }
    };
        
    
    // --- START ---

    textarea.value = '';
    textarea.onblur();
    
    // in FF strings.WELCOME_MSGs are remembered and put as defaults 
    // for some reason. For that, server sets 'empty' class for
    // inputs with no value...
    if (author.getAttribute('class') == 'empty') {
        author.value = '';
        author.onblur();
    }
    if (email.getAttribute('class') == 'empty') {
        email.value = '';
        email.onblur();
    }
    
    lightboxShade.onclick();    //???
    
    
    // --- COMMUNICATION
    
    function sendPost(e) { 
        if (textarea.className == 'empty' || !textarea.value.trim()) {
            alert(strings.EMPTY_ERR_MSG);
            e.preventDefault();
            return;
        }
        
        if (author.value.length > 32 ) {
            alert(strings.NAME_TOO_LONG);
            e.preventDefault();
            return;
        }
        
        if (email.value.length > 32 ) {
            alert(strings.EMAIL_TOO_LONG);
            e.preventDefault();
            return;
        }
        
        if (author.className == 'empty') {author.value = '';}
        if (email.className == 'empty') {email.value = '';}
        
        // build request's body
        var sBody, sParam, aBody = [];
        for (var i=0, el; el = this.elements[i]; ++i) {
            sParam = encodeURIComponent(el.name);
            sParam += '=';
            sParam += encodeURIComponent(el.value);
            aBody.push(sParam);
        }
        sBody = aBody.join('&');

        oWall.keepOn(sBody) 
        
        // clear editing area
        textarea.value = '';
        textarea.onblur();
        author.onblur();
        email.onblur();
        
        editModeOff();
        
        // we don't want to submit the form
        e.preventDefault();

    };
    
    // onsubmit
    addEvent(document.forms.shout,'submit', sendPost, false);
    
    // start polling
    oWall.keepOn();
    
    // --- JAM LEAKS
    
    addEvent(window, 'unload', function() {

        // unleak prop
        purge(textarea);
        purge(addNote);
        purge(author);
        purge(email)

    });
    
}, false);
