String.prototype.trim = function () {
    return this.replace(/^\s+|\s+$/g, "");
}; 

function getFontSetter(element, fontMin, fontMax) {
    var SAVETY_MARGIN = 5;

    if (typeof element.clientHeight == 'undefined') { return null; }

    fontMin = parseFloat(fontMin);
    if (isNaN(fontMin) || fontMin < 0) { return null; }
    
    fontMax = parseFloat(fontMax);
    if (isNaN(fontMax) || fontMax < 0) { return null; }
    
    var fontSetter = {
        FONT_MIN : fontMin,
        FONT_MAX : fontMax
    };

    var ruler = document.createElement('div');
 
    ruler.style.position = 'absolute';
    // position 4 ie
    ruler.style.top = '0px';
    ruler.style.left = '0px';
    ruler.style.visibility = 'hidden';
    
    var computed = null;
    if (element.nodeType == 1) {    // instanceof Element
        // FF won't recognize stuff like 'font' or 'padding'
        var stylings = ['font-size', 'font-family', 'font-weight',
            'padding-top', 'padding-bottom', 'padding-left', 'padding-right',
            'letter-spacing', 'word-spacing'];
        
        computed = window.getComputedStyle(element,'');
        var re = /(\-([a-z]){1})/g;      
        for (var i = 0, prop; prop = stylings[i]; ++i) {
            var jsProp = prop.replace(re, function () {
                return arguments[2].toUpperCase();
            });
            ruler.style[jsProp] = computed.getPropertyValue(prop);
        }

        // relative line height
        var lineHeightPx = parseInt(computed.getPropertyValue('line-height'));
        var fontSize = parseInt(computed.getPropertyValue('font-size'));
        
        ruler.style.lineHeight = (lineHeightPx / fontSize) + 'em';
    }
    else if (typeof element.style == 'object') {
        for (property in element.style) {
            ruler.style[property] = element.style[property];
        }
    }
    else if (typeof element.style == 'string') {
        ruler.style = element.style;
    }
    
    document.body.appendChild(ruler);

    fontSetter._ruler = ruler;
    fontSetter._origin = element;
    
    function computeFont(content) {
        this._ruler.innerHTML = content;
        var fontSize = this.FONT_MAX;
        do {
            this._ruler.style.fontSize = fontSize + 'px';
            --fontSize;
            if (fontSize < this.FONT_MIN) { break; }
        } while ((this._ruler.clientHeight + SAVETY_MARGIN) 
            > this._origin.clientHeight)
        
        return ++fontSize;
    };
    
    if (computed) {
        fontSetter.computeFont = function(content) {
            // O: width eq 0px no matter styles
            // until element is acutally displayed
            var width = computed.getPropertyValue('width');
            if (width == '0px') { return computeFont.call(this) }

            // if the width is computed properly, there'll be no
            // need to compute it again;
            ruler.style.width = width;
            this.computeFont = computeFont;
            
            return this.computeFont(content);
        }
    }
    else { fontSetter.computeFont = computeFont };
    
    return fontSetter;

};